@@ -1,13 +1,17 @@
 use nix::ifaddrs::*;
+use nix::poll::*;
 use nix::sys::socket::AddressFamily;
 use nix::sys::socket::SockaddrLike;
 use nix::sys::socket::SockaddrStorage;
+use nix::sys::time::TimeSpec;
 use std::collections::HashMap;
 use std::fmt;
 use std::net::Ipv4Addr;
 use std::net::Ipv6Addr;
 use std::net::ToSocketAddrs;
+use std::os::unix::io::RawFd;
 use std::str::FromStr;
+use std::time::Duration;
 
 #[derive(Debug, PartialEq, PartialOrd, Hash, Eq, Clone)]
 pub enum Addr {
@@ -57,9 +61,10 @@ impl Ip {
     }
     fn signed_num_representation(&self) -> String {
         match self.address {
-            Addr::V4(x) => { let n = u32::from(x) as i32;
+            Addr::V4(x) => {
+                let n = u32::from(x) as i32;
                 n.to_string()
-            },
+            }
             Addr::V6(x) => {
                 let n = u128::from(x) as i128;
                 n.to_string()
@@ -112,27 +117,70 @@ pub fn parse_mask(mask: &str) -> Option<u32> {
     }
 }
 
-pub fn parse_v6(address: &str, reverse: bool) -> Option<Addr> {
-    match Ipv6Addr::from_str(address) {
-        Ok(mut i) => {
-            if reverse {
-                let mut j = i.octets();
-                j.reverse();
-                for i in 0..j.len() {
-                    j[i] = (( j[i] & 0x0f ) << 4 )| ( j[i] & 0xf0 ) >> 4;
-                }
-
-                i = Ipv6Addr::from(j);
+pub fn parse_v6(address: &str, input_base: Option<i32>, reverse: bool) -> Option<Addr> {
+    match input_base {
+        Some(base) => {
+            if base == 10 {
+                return Some(Addr::V6(Ipv6Addr::from(
+                    match u128::from_str_radix(address, base as u32) {
+                        Ok(y) => y,
+                        Err(e) => {
+                            eprintln!("cannot convert {}: {}", address, e);
+                            return None;
+                        }
+                    },
+                )));
             }
-            Some(Addr::V6(i))
+
+            None
+        }
+        None => match Ipv6Addr::from_str(address) {
+            Ok(mut i) => {
+                if reverse {
+                    let mut j = i.octets();
+                    j.reverse();
+                    for i in &mut j {
+                        *i = ((*i & 0x0f) << 4) | (*i & 0xf0) >> 4;
+                    }
+
+                    i = Ipv6Addr::from(j);
+                }
+                Some(Addr::V6(i))
+            }
+            Err(_) => None,
         },
-        Err(_) => None,
     }
 }
 
 pub fn parse_v4(address: &str, input_base: Option<i32>, reverse: bool) -> Option<Addr> {
     match input_base {
         Some(base) => {
+            let mut address = address;
+            let mut arr: Vec<&str>;
+            let a;
+            if reverse && address.find('.').is_some() {
+                arr = address.split('.').collect();
+                arr.reverse();
+                a = arr.join(".");
+                address = &a;
+            }
+
+            if address.find('.').is_none()
+                && !reverse
+                && input_base.is_some()
+                && input_base.unwrap() != 16
+            {
+                return Some(Addr::V4(Ipv4Addr::from(
+                    match u32::from_str_radix(address, base as u32) {
+                        Ok(y) => y,
+                        Err(e) => {
+                            eprintln!("cannot convert {}: {}", address, e);
+                            return None;
+                        }
+                    },
+                )));
+            }
+
             let parts: Vec<String>;
             let chars: Vec<char>;
 
@@ -174,15 +222,15 @@ pub fn parse_v4(address: &str, input_base: Option<i32>, reverse: bool) -> Option
                     i = Ipv4Addr::from(j);
                 }
                 Some(Addr::V4(i))
-            },
+            }
             Err(_) => None,
         },
     }
 }
 
 pub fn parse_v4_v6(address: &str, input_base: Option<i32>, reverse: bool) -> Option<Addr> {
-    if address.find(':').is_some() {
-        return parse_v6(address, reverse);
+    if address.find(':').is_some() || (address.len() > 20 && input_base.is_some()) {
+        return parse_v6(address, input_base, reverse);
     }
 
     parse_v4(address, input_base, reverse)
@@ -218,6 +266,14 @@ pub fn parse_address_mask(
         });
     }
 
+    for p in ["https://", "http://", "ftp://", "sftp://", "ftps://"] {
+        if a.starts_with(p) {
+            let v: Vec<&str> = a.split( '/' ).collect();
+            arg = v[2];
+            break;
+        }
+    }
+
     let addrs_iter = format!("{}:443", arg).to_socket_addrs();
     let mut buffer: String;
 
@@ -246,6 +302,7 @@ pub fn parse_address_mask(
 pub fn addresses<'a>(
     ip: &'a Ip,
     used: Option<&'a HashMap<Addr, bool>>,
+    mask: Option<u32>,
 ) -> impl std::iter::Iterator<Item = Ip> + 'a {
     let b = broadcast(ip);
     let mut net = network(ip);
@@ -253,9 +310,17 @@ pub fn addresses<'a>(
     std::iter::from_fn(move || {
         if let Addr::V4(mut x) = net.address {
             if let Addr::V4(y) = b.address {
+                let adder = match mask {
+                    Some(x) => {
+                        net.cidr = x;
+                        let base: u32 = 2;
+                        base.pow(32 - x)
+                    }
+                    None => 1,
+                };
                 while u32::from(x) <= u32::from(y) {
                     net = Ip {
-                        address: Addr::V4(Ipv4Addr::from(u32::from(x) + 1)),
+                        address: Addr::V4(Ipv4Addr::from(u32::from(x) + adder)),
                         cidr: net.cidr,
                     };
 
@@ -275,9 +340,16 @@ pub fn addresses<'a>(
                         None => {}
                     }
 
-                    return Some(Ip {
-                        address: Addr::V4(Ipv4Addr::from(u32::from(x) - 1)),
-                        cidr: net.cidr,
+                    return Some(if mask.is_none() {
+                        Ip {
+                            address: Addr::V4(Ipv4Addr::from(u32::from(x) - 1)),
+                            cidr: net.cidr,
+                        }
+                    } else {
+                        network(&Ip {
+                            address: Addr::V4(Ipv4Addr::from(u32::from(x) - 1)),
+                            cidr: net.cidr,
+                        })
                     });
                 }
             }
@@ -285,9 +357,18 @@ pub fn addresses<'a>(
 
         if let Addr::V6(mut x) = net.address {
             if let Addr::V6(y) = b.address {
+                let adder = match mask {
+                    Some(x) => {
+                        net.cidr = x;
+                        let base: u128 = 2;
+                        base.pow(128 - x)
+                    }
+                    None => 1,
+                };
+
                 while u128::from(x) < u128::from(y) {
                     net = Ip {
-                        address: Addr::V6(Ipv6Addr::from(u128::from(x) + 1)),
+                        address: Addr::V6(Ipv6Addr::from(u128::from(x) + adder)),
                         cidr: net.cidr,
                     };
 
@@ -307,9 +388,16 @@ pub fn addresses<'a>(
                         None => {}
                     }
 
-                    return Some(Ip {
-                        address: Addr::V6(Ipv6Addr::from(u128::from(x) - 1)),
-                        cidr: net.cidr,
+                    return Some(if mask.is_none() {
+                        Ip {
+                            address: Addr::V6(Ipv6Addr::from(u128::from(x) - 1)),
+                            cidr: net.cidr,
+                        }
+                    } else {
+                        network(&Ip {
+                            address: Addr::V6(Ipv6Addr::from(u128::from(x) - 1)),
+                            cidr: net.cidr,
+                        })
                     });
                 }
             }
@@ -467,6 +555,97 @@ pub fn subnet(ip: &Ip) -> Ip {
     }
 }
 
+// if addr's network is lower, or addr's broadcast is higher, then not within outside
+pub fn within(net: &Ip, addr: &Ip) -> bool {
+    match (&net.address, &addr.address) {
+        (Addr::V4(_x), Addr::V4(_y)) => {
+            let n = network(net);
+            let b = broadcast(net);
+
+            let tn = network(addr);
+            let tb = broadcast(addr);
+
+            tn.address >= n.address && tb.address <= b.address && net.cidr <= addr.cidr
+        }
+        (Addr::V6(_x), Addr::V6(_y)) => {
+            let n = network(net);
+            let b = broadcast(net);
+
+            let tn = network(addr);
+            let tb = broadcast(addr);
+
+            tn.address >= n.address && tb.address <= b.address && net.cidr <= addr.cidr
+        }
+        (_, _) => false,
+    }
+}
+
+// if addr's network is lower, or addr's broadcast is higher, then not within outside
+pub fn without(net: &Ip, addr: &Ip) -> bool {
+    match (&net.address, &addr.address) {
+        (Addr::V4(_x), Addr::V4(_y)) => {
+            let n = network(net);
+            let b = broadcast(net);
+
+            let tn = network(addr);
+            let tb = broadcast(addr);
+
+            !(tn.address >= n.address && tb.address <= b.address)
+        }
+        (Addr::V6(_x), Addr::V6(_y)) => {
+            let n = network(net);
+            let b = broadcast(net);
+
+            let tn = network(addr);
+            let tb = broadcast(addr);
+
+            !(tn.address >= n.address && tb.address <= b.address)
+        }
+        (_, _) => false,
+    }
+}
+
+// if addr's network is lower, or addr's broadcast is higher, then not within outside
+pub fn withoverlap(net: &Ip, addr: &Ip) -> bool {
+    match (&net.address, &addr.address) {
+        (Addr::V4(_x), Addr::V4(_y)) => {
+            let n = network(net);
+            let b = broadcast(net);
+
+            let tn = network(addr);
+            let tb = broadcast(addr);
+
+            if tn.address <= n.address && tb.address >= n.address {
+                return true;
+            }
+
+            if tn.address <= b.address && tb.address > b.address {
+                return true;
+            }
+
+            false
+        }
+        (Addr::V6(_x), Addr::V6(_y)) => {
+            let n = network(net);
+            let b = broadcast(net);
+
+            let tn = network(addr);
+            let tb = broadcast(addr);
+
+            if tn.address <= n.address && tb.address >= n.address {
+                return true;
+            }
+
+            if tn.address <= b.address && tb.address > b.address {
+                return true;
+            }
+
+            false
+        }
+        (_, _) => false,
+    }
+}
+
 pub fn network_reservation(ip: &Ip) -> Option<String> {
     let mut rows: HashMap<Ip, String> = HashMap::new();
 
@@ -875,7 +1054,7 @@ pub fn format_details(
     formatted: String,
     rows: &Option<HashMap<Ip, NetRow>>,
 ) -> Option<String> {
-    let mut ip = &mut ip.clone();
+    let ip = &mut ip.clone();
     let mut reformatted = formatted;
 
     if rows.is_some() {
@@ -1082,3 +1261,25 @@ pub fn format_details(
 
     Some(out_str)
 }
+
+pub fn fd_ready(fd: RawFd) -> bool {
+    let opfds: Vec<PollFd> = vec![PollFd::new(fd, PollFlags::POLLIN | PollFlags::POLLNVAL)];
+
+    let mut pfds = opfds;
+    let ts = TimeSpec::from(Duration::new(0, 0));
+    match ppoll(&mut pfds, Some(ts), None) {
+        Ok(x) => {
+            if x == 0 {
+                if let Some(ev) = pfds[0].revents() {
+                    if ev | PollFlags::POLLNVAL == PollFlags::POLLNVAL {
+                        return true;
+                    }
+                }
+                false
+            } else {
+                true
+            }
+        }
+        Err(_x) => false,
+    }
+}
